@@ -1,16 +1,17 @@
 use chrono::Utc;
 use flurry::HashMap;
 use pipebuilder_common::{
-    app_workspace,
+    app_workspace, datetime_utc_to_prost_timestamp,
     grpc::{
         build::{
-            builder_server::Builder, BuildMetadataKey, BuildResponse, CancelResponse,
-            GetLogResponse, ScanResponse,
+            builder_server::Builder, BuildCacheMetadata as RpcBuildCacheMetadata, BuildMetadataKey,
+            BuildResponse, CancelBuildResponse, DeleteBuildCacheResponse, GetBuildLogResponse,
+            ScanBuildCacheResponse, ScanBuildResponse,
         },
         repository::repository_client::RepositoryClient,
     },
-    remove_directory, sub_path, Build, BuildMetadata, BuildStatus, LocalBuildContext, Register,
-    PATH_APP,
+    remove_directory, sub_path, Build, BuildCacheMetadata, BuildMetadata, BuildStatus,
+    LocalBuildContext, Register, PATH_APP,
 };
 use std::sync::Arc;
 use tonic::{transport::Channel, Response};
@@ -21,8 +22,10 @@ pub struct BuilderService {
     register: Register,
     repository_client: RepositoryClient<Channel>,
     context: LocalBuildContext,
-    // builds in progress, namespace/id/version -> join handle
+    // builds in progress, (namespace, id, version) -> build thread join handle
     builds: Arc<HashMap<(String, String, u64), tokio::task::JoinHandle<()>>>,
+    // pre-build caches, (namespace, id, target_platform)
+    caches: Arc<HashMap<(String, String, String), BuildCacheMetadata>>,
 }
 
 impl BuilderService {
@@ -38,6 +41,7 @@ impl BuilderService {
             repository_client,
             context,
             builds: Arc::new(HashMap::new()),
+            caches: Arc::new(HashMap::new()),
         }
     }
 }
@@ -53,7 +57,12 @@ impl Builder for BuilderService {
         let namespace = request.namespace;
         let id = request.id;
         let manifest_version = request.manifest_version;
-        info!("build '{}/{}/{}'", namespace, id, manifest_version);
+        info!(
+            namespace = namespace.as_str(),
+            id = id.as_str(),
+            manifest_version = manifest_version,
+            "build"
+        );
         // lock build snapshot with manifest id
         // update latest build version
         let mut register = self.register.clone();
@@ -64,7 +73,13 @@ impl Builder for BuilderService {
         {
             Ok((_, snapshot)) => snapshot,
             Err(err) => {
-                error!("trigger build failed, error: {:#?}", err);
+                error!(
+                    namespace = namespace.as_str(),
+                    id = id.as_str(),
+                    manifest_version = manifest_version,
+                    "trigger build failed, error: {:#?}",
+                    err
+                );
                 return Err(tonic::Status::internal(format!("{:#?}", err)));
             }
         };
@@ -74,7 +89,13 @@ impl Builder for BuilderService {
         let build_context = self.context.to_owned();
         let target_platform = request.target_platform;
         // start build
-        info!("start build '{}/{}/{}'", namespace, id, build_version);
+        info!(
+            namespace = namespace.as_str(),
+            id = id.as_str(),
+            manifest_version = manifest_version,
+            build_version = build_version,
+            "start build"
+        );
         let build = Build::new(
             namespace,
             id,
@@ -87,27 +108,34 @@ impl Builder for BuilderService {
         let lease_id = self.lease_id;
         let register = self.register.to_owned();
         let builds = self.builds.clone();
-        start_build(lease_id, register, builds, build);
+        let caches = self.caches.clone();
+        start_build(lease_id, register, builds, build, caches);
         Ok(Response::new(BuildResponse {
             version: build_version,
         }))
     }
 
-    async fn cancel(
+    async fn cancel_build(
         &self,
-        request: tonic::Request<pipebuilder_common::grpc::build::CancelRequest>,
-    ) -> Result<tonic::Response<pipebuilder_common::grpc::build::CancelResponse>, tonic::Status>
+        request: tonic::Request<pipebuilder_common::grpc::build::CancelBuildRequest>,
+    ) -> Result<tonic::Response<pipebuilder_common::grpc::build::CancelBuildResponse>, tonic::Status>
     {
         let request = request.into_inner();
         let namespace = request.namespace;
         let id = request.id;
         let version = request.build_version;
-        info!("cancel build '{}/{}/{}'", namespace, id, version);
+        info!(
+            namespace = namespace.as_str(),
+            id = id.as_str(),
+            build_version = version,
+            "cancel build"
+        );
         let builds = self.builds.clone();
         let workspace = self.context.workspace.as_str();
+        // stop local build thread
         if !cancel_local_build(builds, namespace.as_str(), id.as_str(), version) {
             return Err(tonic::Status::invalid_argument(format!(
-                "local build not found for '{}/{}/{}'",
+                "local build not found for (namespace = {}, id = {}, build_version = {})",
                 namespace, id, version
             )));
         }
@@ -116,12 +144,15 @@ impl Builder for BuilderService {
         let app_path = sub_path(app_directory.as_str(), PATH_APP);
         if remove_directory(app_path.as_str()).await.is_err() {
             error!(
-                "clean app directory failed for '{}/{}/{}'",
-                namespace, id, version
+                namespace = namespace.as_str(),
+                id = id.as_str(),
+                build_version = version,
+                "clean app directory failed"
             )
         };
         let mut register = self.register.clone();
         let lease_id = self.lease_id;
+        // update metadata
         match cancel_build(
             &mut register,
             lease_id,
@@ -131,7 +162,7 @@ impl Builder for BuilderService {
         )
         .await
         {
-            Ok(_) => Ok(Response::new(CancelResponse {})),
+            Ok(_) => Ok(Response::new(CancelBuildResponse {})),
             Err(err) => Err(tonic::Status::internal(format!(
                 "cancel version build failed, error: '{:#?}'",
                 err
@@ -139,10 +170,11 @@ impl Builder for BuilderService {
         }
     }
 
-    async fn scan(
+    async fn scan_build(
         &self,
-        _request: tonic::Request<pipebuilder_common::grpc::build::ScanRequest>,
-    ) -> Result<tonic::Response<pipebuilder_common::grpc::build::ScanResponse>, tonic::Status> {
+        _request: tonic::Request<pipebuilder_common::grpc::build::ScanBuildRequest>,
+    ) -> Result<tonic::Response<pipebuilder_common::grpc::build::ScanBuildResponse>, tonic::Status>
+    {
         info!("scan local build");
         let builds_ref = self.builds.pin();
         let builds = builds_ref
@@ -154,28 +186,103 @@ impl Builder for BuilderService {
                 version: build_version.to_owned(),
             })
             .collect::<Vec<BuildMetadataKey>>();
-        Ok(Response::new(ScanResponse { builds }))
+        Ok(Response::new(ScanBuildResponse { builds }))
     }
 
-    async fn get_log(
+    async fn get_build_log(
         &self,
-        request: tonic::Request<pipebuilder_common::grpc::build::GetLogRequest>,
-    ) -> Result<tonic::Response<pipebuilder_common::grpc::build::GetLogResponse>, tonic::Status>
+        request: tonic::Request<pipebuilder_common::grpc::build::GetBuildLogRequest>,
+    ) -> Result<tonic::Response<pipebuilder_common::grpc::build::GetBuildLogResponse>, tonic::Status>
     {
         let request = request.into_inner();
         let namespace = request.namespace;
         let id = request.id;
         let version = request.build_version;
-        info!("get build log for '{}/{}/{}'", namespace, id, version);
+        info!(
+            namespace = namespace.as_str(),
+            id = id.as_str(),
+            build_version = version,
+            "get build log"
+        );
         let log_directory = self.context.log_directory.as_str();
         match Build::read_log(log_directory, namespace.as_str(), id.as_str(), version).await {
-            Ok(buffer) => Ok(Response::new(GetLogResponse { buffer })),
+            Ok(buffer) => Ok(Response::new(GetBuildLogResponse { buffer })),
             Err(err) => Err(tonic::Status::not_found(format!(
-                "build log for '{}/{}/{}' not found, error: '{}'",
-                namespace, id, version, err
+                "build log not found, error: '{}'",
+                err
             ))),
         }
     }
+
+    async fn delete_build_cache(
+        &self,
+        request: tonic::Request<pipebuilder_common::grpc::build::DeleteBuildCacheRequest>,
+    ) -> Result<
+        tonic::Response<pipebuilder_common::grpc::build::DeleteBuildCacheResponse>,
+        tonic::Status,
+    > {
+        let request = request.into_inner();
+        let namespace = request.namespace;
+        let id = request.id;
+        let target_platform = request.target_platform;
+        let restore_directory = self.context.restore_directory.as_str();
+        info!(
+            namespace = namespace.as_str(),
+            id = id.as_str(),
+            target_platform = target_platform.as_str(),
+            "delete build cache"
+        );
+        match Build::delete_build_cache(
+            restore_directory,
+            namespace.as_str(),
+            id.as_str(),
+            target_platform.as_str(),
+        )
+        .await
+        {
+            Ok(_) => {
+                // cleanup build cache key set
+                let caches = self.caches.pin();
+                caches.remove(&(namespace, id, target_platform));
+                Ok(Response::new(DeleteBuildCacheResponse {}))
+            }
+            Err(err) => {
+                error!(
+                    namespace = namespace.as_str(),
+                    id = id.as_str(),
+                    target_platform = target_platform.as_str(),
+                    "delete build cache failed, error: '{:#?}'",
+                    err
+                );
+                Err(tonic::Status::internal(format!(
+                    "delete build cache failed, error: '{:#?}'",
+                    err
+                )))
+            }
+        }
+    }
+
+    async fn scan_build_cache(
+        &self,
+        _request: tonic::Request<pipebuilder_common::grpc::build::ScanBuildCacheRequest>,
+    ) -> Result<
+        tonic::Response<pipebuilder_common::grpc::build::ScanBuildCacheResponse>,
+        tonic::Status,
+    > {
+        let caches = self.caches.pin();
+        let caches = caches
+            .into_iter()
+            .map(
+                |((namespace, id, target_platform), metadata)| RpcBuildCacheMetadata {
+                    namespace: namespace.to_owned(),
+                    id: id.to_owned(),
+                    target_platform: target_platform.to_owned(),
+                    timestamp: Some(datetime_utc_to_prost_timestamp(metadata.get_timestamp())),
+                },
+            )
+            .collect::<Vec<RpcBuildCacheMetadata>>();
+        Ok(Response::new(ScanBuildCacheResponse { caches }))
+    }
 }
 
 fn start_build(
@@ -183,6 +290,7 @@ fn start_build(
     mut register: Register,
     builds: Arc<HashMap<(String, String, u64), tokio::task::JoinHandle<()>>>,
     mut build: Build,
+    caches: Arc<HashMap<(String, String, String), BuildCacheMetadata>>,
 ) {
     let builds_clone = builds.clone();
     let key_tuple = build.get_build_key_tuple();
@@ -193,10 +301,16 @@ fn start_build(
             match update(&mut register, lease_id, &build, status.clone(), None).await {
                 Ok(()) => (),
                 Err(err) => {
-                    let (namespace, id, _, build_version) = build.get_build_meta();
+                    let (namespace, id, manifest_version, build_version, target_platform) =
+                        build.get_build_meta();
                     error!(
-                        "update build status for '{}/{}:{}' fail, error: '{}'",
-                        namespace, id, build_version, err
+                        namespace = namespace.as_str(),
+                        id = id.as_str(),
+                        manifest_version = manifest_version,
+                        build_version = build_version,
+                        target_platform = target_platform.as_str(),
+                        "update build status fail, error: '{:#?}'",
+                        err
                     );
                     break;
                 }
@@ -206,12 +320,15 @@ fn start_build(
             let next_status = match result {
                 Ok(next_status) => next_status,
                 Err(err) => {
-                    let (namespace, id, _, build_version) = build.get_build_meta();
+                    let (namespace, id, manifest_version, build_version, target_platform) =
+                        build.get_build_meta();
                     error!(
-                        "run build for '{}/{}:{}' fail, status: '{}', error: '{}'",
-                        namespace,
-                        id,
-                        build_version,
+                        namespace = namespace.as_str(),
+                        id = id.as_str(),
+                        manifest_version = manifest_version,
+                        build_version = build_version,
+                        target_platform = target_platform.as_str(),
+                        "run build fail, status: '{}', error: '{:#?}'",
                         status.to_string(),
                         err
                     );
@@ -232,9 +349,17 @@ fn start_build(
                 None => break,
             }
         }
-        let key_tuple = build.get_build_key_tuple();
+        let build_key_tuple = build.get_build_key_tuple();
         // remove local build
-        builds_clone.pin().remove(&key_tuple);
+        builds_clone.pin().remove(&build_key_tuple);
+        // update build cache key set if build succeed
+        if matches!(status, BuildStatus::Succeed) {
+            let build_cache_key_tuple = build.get_build_cache_key_tuple();
+            let build_cache_metadata = BuildCacheMetadata::new();
+            caches
+                .pin()
+                .insert(build_cache_key_tuple, build_cache_metadata);
+        }
     });
     // register local build
     builds.pin().insert(key_tuple, jh);
@@ -248,9 +373,8 @@ async fn update(
     status: BuildStatus,
     message: Option<String>,
 ) -> pipebuilder_common::Result<()> {
-    let (namespace, id, _, build_version) = build.get_build_meta();
+    let (namespace, id, _, build_version, target_platform) = build.get_build_meta();
     let (builder_id, builder_address) = build.get_builder_meta();
-    let target_platform = build.get_target_platform();
     let now = Utc::now();
     let version_build = BuildMetadata::new(
         target_platform.to_owned(),
@@ -287,8 +411,10 @@ fn cancel_local_build(
         }
         None => {
             warn!(
-                "cancel non-extists build '{}/{}/{}'",
-                namespace, id, version
+                namespace = namespace,
+                id = id,
+                build_version = version,
+                "cancel non-exists build"
             );
             false
         }
@@ -309,8 +435,10 @@ async fn cancel_build(
         Some(version_build) => version_build,
         None => {
             warn!(
-                "cancel non-extists build '{}/{}/{}'",
-                namespace, id, version
+                namespace = namespace,
+                id = id,
+                build_version = version,
+                "cancel non-extists build"
             );
             return Ok(());
         }
