@@ -10,7 +10,7 @@ use pipebuilder_common::{
         health::health_server::HealthServer, node::node_server::NodeServer,
         repository::repository_server::RepositoryServer,
     },
-    open_file, parse_config, Result, ENV_PIPEBUILDER_CONFIG_FILE,
+    init_tracing_subscriber, open_file, parse_config, Result, ENV_PIPEBUILDER_CONFIG_FILE,
 };
 use std::net::SocketAddr;
 use tonic::transport::Server;
@@ -19,7 +19,7 @@ use tracing::{info, instrument};
 #[tokio::main]
 #[instrument]
 async fn main() -> Result<()> {
-    tracing_subscriber::fmt::init();
+    init_tracing_subscriber();
     info!("read configuration ...");
     let file = open_file(std::env::var(ENV_PIPEBUILDER_CONFIG_FILE)?).await?;
     let config = parse_config::<Config>(file).await?;
@@ -34,8 +34,9 @@ async fn main() -> Result<()> {
     let internal_address = node_svc.get_internal_address();
     let addr: SocketAddr = internal_address.parse()?;
     info!(
-        "run repository server {:?}, internal address {:?}...",
-        node_id, internal_address
+        node_id = node_id.as_str(),
+        internal_address = internal_address.as_str(),
+        "run repository server ..."
     );
     Server::builder()
         .add_service(HealthServer::new(health_svc))
@@ -43,6 +44,6 @@ async fn main() -> Result<()> {
         .add_service(NodeServer::new(node_svc))
         .serve_with_shutdown(addr, shutdown_rx.map(drop))
         .await?;
-    info!("repository server {:?} exit ...", node_id);
+    info!(node_id = node_id.as_str(), "repository server exit ...");
     Ok(())
 }
